Method: MemCache#reset

Defined in:
lib/memcache.rb

#resetObject

Reset the connection to all memcache servers. This should be called if there is a problem with a cache lookup that might have left the connection in a corrupted state.



574
575
576
# File 'lib/memcache.rb', line 574

def reset
  @servers.each { |server| server.close }
end