Class: MemCache::Server

Inherits:
Object
  • Object
show all
Defined in:
lib/memcache.rb

Overview

This class represents a memcached server instance.

Constant Summary collapse

RETRY_DELAY =

The amount of time to wait before attempting to re-establish a connection with a server that is marked dead.

30.0

Instance Attribute Summary collapse

Instance Method Summary collapse

Constructor Details

#initialize(memcache, host, port = DEFAULT_PORT, weight = DEFAULT_WEIGHT) ⇒ Server

Create a new MemCache::Server object for the memcached instance listening on the given host and port, weighted by the given weight.

Raises:

  • (ArgumentError)


959
960
961
962
963
964
965
966
967
968
969
970
971
972
# File 'lib/memcache.rb', line 959

def initialize(memcache, host, port = DEFAULT_PORT, weight = DEFAULT_WEIGHT)
  raise ArgumentError, "No host specified" if host.nil? or host.empty?
  raise ArgumentError, "No port specified" if port.nil? or port.to_i.zero?

  @host   = host
  @port   = port.to_i
  @weight = weight.to_i

  @sock   = nil
  @retry  = nil
  @status = 'NOT CONNECTED'
  @timeout = memcache.timeout
  @logger = memcache.logger
end

Instance Attribute Details

#hostObject (readonly)

The host the memcached server is running on.



931
932
933
# File 'lib/memcache.rb', line 931

def host
  @host
end

#loggerObject (readonly)

Returns the value of attribute logger.



953
954
955
# File 'lib/memcache.rb', line 953

def logger
  @logger
end

#portObject (readonly)

The port the memcached server is listening on.



936
937
938
# File 'lib/memcache.rb', line 936

def port
  @port
end

#retryObject (readonly)

The time of next retry if the connection is dead.



946
947
948
# File 'lib/memcache.rb', line 946

def retry
  @retry
end

#statusObject (readonly)

A text status string describing the state of the server.



951
952
953
# File 'lib/memcache.rb', line 951

def status
  @status
end

#weightObject (readonly)

The weight given to the server.



941
942
943
# File 'lib/memcache.rb', line 941

def weight
  @weight
end

Instance Method Details

#alive?Boolean

Check whether the server connection is alive. This will cause the socket to attempt to connect if it isn’t already connected and or if the server was previously marked as down and the retry time has been exceeded.

Returns:

  • (Boolean)


987
988
989
# File 'lib/memcache.rb', line 987

def alive?
  !!socket
end

#closeObject

Close the connection to the memcached server targeted by this object. The server is not considered dead.



1048
1049
1050
1051
1052
1053
# File 'lib/memcache.rb', line 1048

def close
  @sock.close if @sock && !@sock.closed?
  @sock   = nil
  @retry  = nil
  @status = "NOT CONNECTED"
end

#connect_to(host, port, timeout = nil) ⇒ Object



1017
1018
1019
1020
1021
1022
1023
1024
1025
1026
1027
1028
1029
1030
1031
1032
1033
1034
1035
1036
1037
1038
1039
1040
1041
1042
# File 'lib/memcache.rb', line 1017

def connect_to(host, port, timeout=nil)
  s = TCPSocket.new(host, port, 0)
  if timeout
    s.instance_eval "      alias :blocking_gets :gets\n      def gets(*args)\n        MemCacheTimer.timeout(\#{timeout}) do\n          self.blocking_gets(*args)\n        end\n      end\n      alias :blocking_read :read\n      def read(*args)\n        MemCacheTimer.timeout(\#{timeout}) do\n          self.blocking_read(*args)\n        end\n      end\n      alias :blocking_write :write\n      def write(*args)\n        MemCacheTimer.timeout(\#{timeout}) do\n          self.blocking_write(*args)\n        end\n      end\n    EOR\n  end\n  s\nend\n"

#inspectObject

Return a string representation of the server object.



977
978
979
# File 'lib/memcache.rb', line 977

def inspect
  "<MemCache::Server: %s:%d [%d] (%s)>" % [@host, @port, @weight, @status]
end

#mark_dead(error) ⇒ Object

Mark the server as dead and close its socket.



1058
1059
1060
1061
1062
1063
1064
1065
1066
# File 'lib/memcache.rb', line 1058

def mark_dead(error)
  @sock.close if @sock && !@sock.closed?
  @sock   = nil
  @retry  = Time.now + RETRY_DELAY

  reason = "#{error.class.name}: #{error.message}"
  @status = sprintf "%s:%s DEAD (%s), will retry at %s", @host, @port, reason, @retry
  @logger.info { @status } if @logger
end

#socketObject

Try to connect to the memcached server targeted by this object. Returns the connected socket object on success or nil on failure.



995
996
997
998
999
1000
1001
1002
1003
1004
1005
1006
1007
1008
1009
1010
1011
1012
1013
1014
1015
# File 'lib/memcache.rb', line 995

def socket
  return @sock if @sock and not @sock.closed?

  @sock = nil

  # If the host was dead, don't retry for a while.
  return if @retry and @retry > Time.now

  # Attempt to connect if not already connected.
  begin
    @sock = connect_to(@host, @port, @timeout)
    @sock.setsockopt Socket::IPPROTO_TCP, Socket::TCP_NODELAY, 1
    @retry  = nil
    @status = 'CONNECTED'
  rescue SocketError, SystemCallError, IOError => err
    logger.warn { "Unable to open socket: #{err.class.name}, #{err.message}" } if logger
    mark_dead err
  end

  return @sock
end